package com.mapmyindia.sdk.flutter.nearby.mapmyindia_nearby_plugin

import android.app.Activity
import android.content.Intent
import androidx.annotation.NonNull
import com.google.gson.Gson
import com.mapmyindia.sdk.maps.MapmyIndia
import com.mapmyindia.sdk.nearby.plugin.CategoryCode
import com.mapmyindia.sdk.nearby.plugin.MapmyIndiaNearbyWidget
import com.mapmyindia.sdk.nearby.plugin.model.NearbyOption
import com.mapmyindia.sdk.nearby.plugin.model.NearbyUIOption
import com.mapmyindia.sdk.plugins.places.placepicker.PlacePicker

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry

/** MapmyindiaNearbyPlugin */
class MapmyindiaNearbyPlugin : FlutterPlugin, MethodCallHandler, ActivityAware, PluginRegistry.ActivityResultListener {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel
    private lateinit var result: Result
    private var activity: Activity? = null
    private var binding: ActivityPluginBinding? = null

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "mapmyindia_nearby_plugin")
        channel.setMethodCallHandler(this)
        MapmyIndia.getInstance(flutterPluginBinding.applicationContext)

    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        if (call.method == "openNearbyWidget") {
            if (activity != null) {
                this.result = result
                val builder = MapmyIndiaNearbyWidget.IntentBuilder()
                if (call.hasArgument("nearbyOption")) {
                    val nearbyOption: NearbyOption = Utils.getNearbyOptions(call.argument<Map<String, Any?>>("nearbyOption") as Map<String, Any?>)
                    builder.nearbyOptions(nearbyOption)
                }
                if(call.hasArgument("nearbyUIOption")) {
                    val nearbyUiOption: NearbyUIOption = Utils.getNearbyUIOption(call.argument<Map<String, Any?>>("nearbyUIOption") as Map<String, Any?>)
                    builder.nearbyUIOptions(nearbyUiOption)
                }
                if(call.hasArgument("categories")) {
                    val categoriesList: List<Map<String, Any?>?>? = call.argument<List<Map<String, Any?>?>>("categories")
                    val category: MutableList<CategoryCode> = mutableListOf()
                    categoriesList?.forEach {
                        category.add(Utils.getCategoryCode(it!!))
                    }
                    builder.setCategoryList(category)
                }
                val intent = builder.build(activity!!)
                activity?.startActivityForResult(intent, 12)
                binding?.addActivityResultListener(this)
            } else {
                result.error("Failure", "Not attached to activity", null)
            }
        } else {
            result.notImplemented()
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {

        this.activity = binding.activity
        this.binding = binding;

    }

    override fun onDetachedFromActivityForConfigChanges() {
        this.activity = null
        this.binding = null
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        this.activity = binding.activity
        this.binding = binding
    }

    override fun onDetachedFromActivity() {

        this.activity = null
        this.binding = null
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
        if (requestCode == 12) {
            if (resultCode == Activity.RESULT_OK) {
                result.success(Gson().toJson(MapmyIndiaNearbyWidget.getNearbyResponse(data!!)))
                binding?.removeActivityResultListener(this)

            }
        }
        return true
    }


}
