import UIKit
import MapmyIndiaNearbyUI
import MapmyIndiaAPIKit
import MapmyIndiaMaps
class MapmyIndiaNearbyFlutterConfiguration: NSObject {
    
    var nearbyFilterConfiguration = MapmyIndiaNearbyFilterConfiguration()
    var nearbyCategoryConfiguration = MapmyIndiaNearbyCategoryConfiguration()
    
    func setNearbyConfiguration(argsLists : [String : Any], nearbyPlugin: MapmyIndiaNearbyCategoriesViewController) {
        let configuration = argsLists
        
        if configuration.keys.contains("nearbyOption") {
            let nearbyFilterConfiguration = configuration["nearbyOption"]
            let filterConfiguration =  setMapmyIndiaNearbyFilterConfiguration(filterConfiguration: nearbyFilterConfiguration as! [String : Any])
            nearbyPlugin.mapmyIndiaNearbyConfiguration.mapmyIndiaNearbyFilterConfiguration = filterConfiguration
        }
        
        if configuration.keys.contains("nearbyUIOption") {
            let nearbyUIConfiguration = configuration["nearbyUIOption"]
            let neabyUIConfig = nearbyUIConfiguration as! [String: Any]
            let uiConfiguratin = setMapmyIndiaNearbyUIConfiguration(mapConfiguration: neabyUIConfig)
            
            if neabyUIConfig.keys.contains("backgroundColor") {
                nearbyPlugin.view.backgroundColor = hexStringToUIColor(hex: neabyUIConfig["backgroundColor"] as! String)
            }
            
            nearbyPlugin.mapmyIndiaNearbyConfiguration.mapmyIndiaNearbyUIConfiguration = uiConfiguratin
            
            let categoryConfiguratin = setMapmyIndiaNearbyCatageoryConfiguration(categoryConfiguration: neabyUIConfig)
            nearbyPlugin.mapmyIndiaNearbyCategoryConfiguration = categoryConfiguratin
        }
        
        if configuration.keys.contains("categories") {
            var mapmyIndiaNearbyCategories = [MapmyIndiaNearbyCategories]()
            let categries = configuration["categories"] as! Array<Any>
            for i in categries {
                let list = i as! [String: Any]
                let image = imageConversion(image: list["icon"] as! FlutterStandardTypedData)
                let categoryCodeArray = list["categoryCode"] as! Array<String>
                let mapNearbyCategoryIcon = imageConversion(image: list["markerIcon"] as! FlutterStandardTypedData)
                
                let category = MapmyIndiaNearbyCategories(title: list["category"] as! String, selectedImage: image, isSelected: list["isSelected"] as? Bool ?? true , categoryKeywords: categoryCodeArray, mapNearbyCategoryIcon: mapNearbyCategoryIcon)

                mapmyIndiaNearbyCategories.append(category)
            }
            
            nearbyPlugin.mapmyIndiaNearbyCategories = mapmyIndiaNearbyCategories
        }
    }
    
    func setMapmyIndiaNearbyFilterConfiguration(filterConfiguration:[String:Any]) -> MapmyIndiaNearbyFilterConfiguration  {
        
        if filterConfiguration.keys.contains("refLocation") {
            nearbyFilterConfiguration.refLocation = filterConfiguration["refLocation"] as? String
        }
        if filterConfiguration.keys.contains("radius") {
            nearbyFilterConfiguration.radius = filterConfiguration["radius"] as? Int ?? 1000
        }
        if filterConfiguration.keys.contains("sortBy") {
            if filterConfiguration["sortBy"] as? String == "dist:asc" || filterConfiguration["sortBy"] as? String == "dist:desc" {
                nearbyFilterConfiguration.sortBy = MapmyIndiaSortByDistanceWithOrder(orderBy: MMISortByOrderType.init(filterConfiguration["sortBy"] as? String ?? "dist:asc"))
            }
            else if filterConfiguration["sortBy"] as? String == "name:asc" || filterConfiguration["sortBy"] as? String == "name:desc" {
//                nearbyFilterConfiguration.sortBy = MapmyIndiaSortBy(orderBy: MMISortByName.init(filterConfiguration["sortBy"] as? String ?? "name:desc"))
            }
        }
        
        if filterConfiguration.keys.contains("searchBy") {
            let searchBy = MMISearchByType.init(rawValue: filterConfiguration["searchBy"] as? String ?? "")
            nearbyFilterConfiguration.searchBy = searchBy
        }
        
        if filterConfiguration.keys.contains("filter") {
//            model:FO0f;brandId:CHKOPDD;plugType:FJFOSPS
            let filterArray = (filterConfiguration["filter"] as! String).components(separatedBy: ":")
            nearbyFilterConfiguration.filter =  [MapmyIndiaNearbyKeyValueFilter(filterKey: filterArray[0], filterValues: [filterArray[1]])]
                  
        }
        
        
        if filterConfiguration.keys.contains("bounds") {
            let bound = filterConfiguration["bounds"] as? String
            let coordinates = bound?.split(separator: ";")
            if let topLeft = converToCoordinate(location: String(coordinates?.first ?? "")), let bottomRight = converToCoordinate(location: String(coordinates?.last ?? ""))  {
                nearbyFilterConfiguration.bounds = MapmyIndiaRectangularRegion(topLeft: topLeft, bottomRight: bottomRight)
            }
        }
        
        if filterConfiguration.keys.contains("richData") {
            nearbyFilterConfiguration.isRichData = filterConfiguration["richData"] as? Bool ?? false
        }
        if filterConfiguration.keys.contains("userName") {
            nearbyFilterConfiguration.userName = filterConfiguration["userName"] as? String
        }
        if filterConfiguration.keys.contains("pod") {
            let pod = MMIPodTypeIdentifier.init(filterConfiguration["pod"] as? String ?? "SLC")
            nearbyFilterConfiguration.pod = pod
        }
        if filterConfiguration.keys.contains("explain") {
            nearbyFilterConfiguration.shouldExplain = filterConfiguration["explain"] as? Bool ?? false
        }
        return nearbyFilterConfiguration
    }
    
    func setMapmyIndiaNearbyUIConfiguration(mapConfiguration:[String:Any]) -> MapmyIndiaNearbyUIConfiguration {
        let nearbyUIConfiguration = MapmyIndiaNearbyUIConfiguration()
        
        if mapConfiguration.keys.contains("circleAlpha") {
            nearbyUIConfiguration.circleAlpha = mapConfiguration["circleAlpha"] as! CGFloat
        } else {
            nearbyUIConfiguration.circleAlpha = 0.22
        }
        if mapConfiguration.keys.contains("refLocationCircleColor") {
            nearbyUIConfiguration.refLocationCircleColor = hexStringToUIColor(hex: mapConfiguration["refLocationCircleColor"] as! String)
        }
        if mapConfiguration.keys.contains("distanceTextColor") {
            nearbyUIConfiguration.distanceTextColor = hexStringToUIColor(hex: mapConfiguration["distanceTextColor"] as! String)
        }
        if mapConfiguration.keys.contains("navigationBackgroundColor") {
            nearbyUIConfiguration.navigationBackgroundColor = hexStringToUIColor(hex: mapConfiguration["navigationBackgroundColor"] as! String)
        }
        if mapConfiguration.keys.contains("navigationTitleColor") {
                    nearbyUIConfiguration.navigationTitleColor = hexStringToUIColor(hex: mapConfiguration["navigationTitleColor"] as! String)
                }
        
        if mapConfiguration.keys.contains("nearbyFilterCellBorderColor") {
            nearbyUIConfiguration.nearbyFilterCellBorderColor = hexStringToUIColor(hex: mapConfiguration["nearbyFilterCellBorderColor"] as! String)
        }
        if mapConfiguration.keys.contains("nearbyFilterCellBorderWidth") {
            nearbyUIConfiguration.nearbyFilterCellBorderWidth =  mapConfiguration["nearbyFilterCellBorderWidth"] as! Double
        }
        if mapConfiguration.keys.contains("nearbyFilterCellCornerRadius") {
            nearbyUIConfiguration.nearbyFilterCellCornerRadius = mapConfiguration["nearbyFilterCellCornerRadius"] as! CGFloat
        }
        if mapConfiguration.keys.contains("nearbyFilterViewContainerBackgroundColor") {
            nearbyUIConfiguration.nearbyFilterViewContainerBackgroundColor = hexStringToUIColor(hex: mapConfiguration["nearbyFilterViewContainerBackgroundColor"] as! String)
        }
        if mapConfiguration.keys.contains("nearbySegmentedViewContainerBackgroundColor") {
            nearbyUIConfiguration.nearbySegmentedViewContainerBackgroundColor = hexStringToUIColor(hex: mapConfiguration["nearbySegmentedViewContainerBackgroundColor"] as! String)
        }
        if mapConfiguration.keys.contains("paginationContainerViewBackgroundColor") {
            nearbyUIConfiguration.paginationContainerViewBackgroundColor = hexStringToUIColor(hex: mapConfiguration["paginationContainerViewBackgroundColor"] as! String)
        }
        if mapConfiguration.keys.contains("paginationNextButtonImage") {
            nearbyUIConfiguration.paginationNextButtonImage = imageConversion(image: mapConfiguration["paginationNextButtonImage"] as! FlutterStandardTypedData)
            
        }
        if mapConfiguration.keys.contains("paginationNextButtonbackgroundcolor") {
            nearbyUIConfiguration.paginationNextButtonbackgroundcolor = hexStringToUIColor(hex: mapConfiguration["paginationNextButtonbackgroundcolor"] as! String)
        }
        if mapConfiguration.keys.contains("paginationPreviousButtonImage") {
            nearbyUIConfiguration.paginationPreviousButtonImage = imageConversion(image: mapConfiguration["paginationPreviousButtonImage"] as! FlutterStandardTypedData)
        }
        if mapConfiguration.keys.contains("paginationPreviousButtonbackgroundcolor") {
            nearbyUIConfiguration.paginationPreviousButtonbackgroundcolor = hexStringToUIColor(hex: mapConfiguration["paginationPreviousButtonbackgroundcolor"] as! String)
        }
        if mapConfiguration.keys.contains("placeAddressTextColor") {
            nearbyUIConfiguration.placeAddressTextColor = hexStringToUIColor(hex: mapConfiguration["placeAddressTextColor"] as! String)
        }
        if mapConfiguration.keys.contains("placeNameTextColor") {
            nearbyUIConfiguration.placeNameTextColor = hexStringToUIColor(hex: mapConfiguration["placeNameTextColor"] as! String)
        }
        if mapConfiguration.keys.contains("refLocationIcon") {
            nearbyUIConfiguration.refLocationIcon = imageConversion(image: mapConfiguration["refLocationIcon"] as! FlutterStandardTypedData)
        }
        if mapConfiguration.keys.contains("segmentedControlBackgroundColor") {
            nearbyUIConfiguration.segmentedControlBackgroundColor = hexStringToUIColor(hex: mapConfiguration["segmentedControlBackgroundColor"] as! String)
        }
        if mapConfiguration.keys.contains("segmentedControlBorderColor") {
            nearbyUIConfiguration.segmentedControlBorderColor = hexStringToUIColor(hex: mapConfiguration["segmentedControlBorderColor"] as! String)
        }
        if mapConfiguration.keys.contains("segmentedControlBorderWidth") {
            nearbyUIConfiguration.segmentedControlBorderWidth = mapConfiguration["segmentedControlBorderWidth"] as! CGFloat
        }
        if mapConfiguration.keys.contains("segmentedControlSelectedSegmentIndex") {
            nearbyUIConfiguration.segmentedControlSelectedSegmentIndex = mapConfiguration["segmentedControlSelectedSegmentIndex"] as! Int
        }
        if mapConfiguration.keys.contains("segmentedControlSelectedSegmentTintColor") {
            nearbyUIConfiguration.segmentedControlSelectedSegmentTintColor = hexStringToUIColor(hex: mapConfiguration["segmentedControlSelectedSegmentTintColor"] as! String)
        }
        if mapConfiguration.keys.contains("segmentedControlSelectedforegroundColor") {
            nearbyUIConfiguration.segmentedControlSelectedforegroundColor = hexStringToUIColor(hex: mapConfiguration["segmentedControlSelectedforegroundColor"] as! String)
        }
        if mapConfiguration.keys.contains("segmentedControlUnselectedforegroundColor") {
            nearbyUIConfiguration.segmentedControlUnselectedforegroundColor = hexStringToUIColor(hex: mapConfiguration["segmentedControlUnselectedforegroundColor"] as! String)
        } else {
            nearbyUIConfiguration.segmentedControlUnselectedforegroundColor = hexStringToUIColor(hex: "#DC4929")
        }
        if mapConfiguration.keys.contains("segmentedControltTextFont") {
            nearbyUIConfiguration.segmentedControltTextFont = mapConfiguration["segmentedControltTextFont"] as! UIFont
        }
        if mapConfiguration.keys.contains("tableCellBackgroundColor") {
            nearbyUIConfiguration.tableCellBackgroundColor = hexStringToUIColor(hex: mapConfiguration["tableCellBackgroundColor"] as! String)
        }
        if mapConfiguration.keys.contains("tableCellSeparatorColor") {
            nearbyUIConfiguration.tableCellSeparatorColor = hexStringToUIColor(hex: mapConfiguration["tableCellSeparatorColor"] as! String)
        }
        
        return nearbyUIConfiguration
    }
    
    func setMapmyIndiaNearbyCatageoryConfiguration(categoryConfiguration:[String:Any]) -> MapmyIndiaNearbyCategoryConfiguration {
        
        if categoryConfiguration.keys.contains("changeLocationButtonTextColor") {
            nearbyCategoryConfiguration.changeLocationButtonTitleColor = hexStringToUIColor(hex:  categoryConfiguration["changeLocationButtonTextColor"] as! String)
        }
        
        if categoryConfiguration.keys.contains("useCurrentLocationButtonTextColor") {
            nearbyCategoryConfiguration.useCurrentLocationButtonTitleColor = hexStringToUIColor(hex:  categoryConfiguration["useCurrentLocationButtonTextColor"] as! String)
        }
        
        if categoryConfiguration.keys.contains("locationDetailFormattedAddressTextColor") {
            nearbyCategoryConfiguration.locationDetailsFormatedAddressLabelTextColor = hexStringToUIColor(hex:  categoryConfiguration["locationDetailFormattedAddressTextColor"] as! String)
        }
        
        if categoryConfiguration.keys.contains("locationDetailsInfoLabelText") {
            nearbyCategoryConfiguration.locationDetailsInfoLabelText = categoryConfiguration["locationDetailsInfoLabelText"] as! String
        }
        
        if categoryConfiguration.keys.contains("locationDetailInfoTextColor") {
            nearbyCategoryConfiguration.locationDetailsInfoLabelTextColor = hexStringToUIColor(hex:  categoryConfiguration["locationDetailInfoTextColor"] as! String)
        }
        
        if categoryConfiguration.keys.contains("locationDetailsBackgroundColor") {
            nearbyCategoryConfiguration.locationDetailsViewBackground = hexStringToUIColor(hex:  categoryConfiguration["locationDetailsBackgroundColor"] as! String)
        }
        
        if categoryConfiguration.keys.contains("nearbyToolbarIcon") {
            nearbyCategoryConfiguration.navigationImage = imageConversion(image: categoryConfiguration["nearbyToolbarIcon"] as! FlutterStandardTypedData)
        }
        
        if categoryConfiguration.keys.contains("nearbyToolbarColor") {
            nearbyCategoryConfiguration.titleViewContainerBackgroundColor = hexStringToUIColor(hex: categoryConfiguration["nearbyToolbarColor"] as! String)
        }
        if categoryConfiguration.keys.contains("nearbyToolbarTintColor") {
            nearbyCategoryConfiguration.titleColor = hexStringToUIColor(hex: categoryConfiguration["nearbyToolbarTintColor"] as! String)
        }
        if categoryConfiguration.keys.contains("submitButtonColor") {
            nearbyCategoryConfiguration.nextButtonBackgroundColor = hexStringToUIColor(hex:  categoryConfiguration["submitButtonColor"] as! String)
        }
        
        if categoryConfiguration.keys.contains("selectedCategoryTextColor") {
            nearbyCategoryConfiguration.selectedCategoriesTitleColor = hexStringToUIColor(hex:  categoryConfiguration["selectedCategoryTextColor"] as! String)
        }
        
        if categoryConfiguration.keys.contains("categoryTextColor") {
            nearbyCategoryConfiguration.unselectedCategoriesTitleColor = hexStringToUIColor(hex: categoryConfiguration["categoryTextColor"] as! String)
        }
        
        if categoryConfiguration.keys.contains("selectedCategoryBackgroundColor") {
            nearbyCategoryConfiguration.selectedCategoriesBackgroundColor = hexStringToUIColor(hex:  categoryConfiguration["selectedCategoryBackgroundColor"] as! String)
        }
        
        if categoryConfiguration.keys.contains("categoryBackgroundColor") {
            nearbyCategoryConfiguration.unselectedCategoriesBackgroundColor = hexStringToUIColor(hex:  categoryConfiguration["categoryBackgroundColor"] as! String)
        }
        
        if categoryConfiguration.keys.contains("categoryIconColor") {
            nearbyCategoryConfiguration.categoryUnselectedIconColor = hexStringToUIColor(hex:  categoryConfiguration["categoryIconColor"] as! String)
        }
        
        if categoryConfiguration.keys.contains("selectedCategoryIconColor") {
            nearbyCategoryConfiguration.categorySelectedIconColor = hexStringToUIColor(hex:  categoryConfiguration["selectedCategoryIconColor"] as! String)
        }
        
        if categoryConfiguration.keys.contains("submitButtonColor") {
            nearbyCategoryConfiguration.nextButtonBackgroundColor = hexStringToUIColor(hex: categoryConfiguration["submitButtonColor"] as! String)
        }
        
        if categoryConfiguration.keys.contains("submitButtonText") {
            nearbyCategoryConfiguration.nextButtonTitle = categoryConfiguration["submitButtonText"] as! String
        }
        
        if categoryConfiguration.keys.contains("submitButtonTextColor") {
            nearbyCategoryConfiguration.nextButtonTextColor = hexStringToUIColor(hex: categoryConfiguration["submitButtonTextColor"] as! String)
        }
        return nearbyCategoryConfiguration
    }
    
}

func imageConversion(image: FlutterStandardTypedData) -> UIImage {
    let marker = image
    let myData = Data(marker.data)
    let myUInt8bytes: [UInt8] = myData.toArray(type: UInt8.self)
    let data = NSData(bytes: myUInt8bytes, length: myUInt8bytes.count)
    let image = UIImage(data: data as Data)?.withRenderingMode(.alwaysTemplate)
    return image ?? UIImage()
    
}

extension Data {
    
    init<T>(fromArray values: [T]) {
        var values = values
        self.init(buffer: UnsafeBufferPointer(start: &values, count: values.count))
    }
    
    func toArray<T>(type: T.Type) -> [T] {
        return self.withUnsafeBytes {
            [T](UnsafeBufferPointer(start: $0, count: self.count/MemoryLayout<T>.stride))
        }
    }
}

func converToCoordinate(location: String) -> CLLocationCoordinate2D? {
   let locationString = location.split(separator: ",")
   if locationString.count < 2 {
       return nil
   } else if locationString.count == 2 {
       let latitude: Double = Double(locationString.first!) ?? 0.0
       let longitue: Double = Double(locationString.last!) ?? 0.0
       let coordinate = CLLocationCoordinate2D(latitude: CLLocationDegrees(latitude), longitude: CLLocationDegrees(longitue))
       print("coordinate: \(latitude), \(longitue)")
       return coordinate
   } else {
       return nil
   }
}

func hexStringToUIColor (hex:String) -> UIColor {
    var cString:String = hex.trimmingCharacters(in: .whitespacesAndNewlines).uppercased()
    
    if (cString.hasPrefix("#")) {
        cString.remove(at: cString.startIndex)
    }
    
    if ((cString.count) != 6) {
        return UIColor.gray
    }
    
    var rgbValue:UInt64 = 0
    Scanner(string: cString).scanHexInt64(&rgbValue)
    
    return UIColor(
        red: CGFloat((rgbValue & 0xFF0000) >> 16) / 255.0,
        green: CGFloat((rgbValue & 0x00FF00) >> 8) / 255.0,
        blue: CGFloat(rgbValue & 0x0000FF) / 255.0,
        alpha: CGFloat(1.0)
    )
}
